<?php

/**
 * @file
 * Defines link type classes for Autolink module.
 */

/**
 * Defines the link collector interface for collecting links for processing.
 */
interface AutolinkCollectorInterface {
  function fetchLinks($node_type);
}

/**
 * Defines the Autolink link collector class for loading link objects for processing.
 */
class AutolinkCollector implements AutolinkCollectorInterface {
  private $_info = array();
  protected $_reset = FALSE;
  protected $maker;

  function __construct() {
    $this->_info = autolink_get('PluginInfo')->getCurrent();
  }

  function __destruct() {
    $this->_info = array();
  }

  /**
   * Returns *all* links, defined and abstract, for each node type.
   *
   * @param $cached
   *   A boolean value indicating whether cached links may be returned.
   *   Defaults to FALSE.
   * @return
   *   An array of link objects, defined and abstract, for all node types.
   */
  function fetchAll($cached = FALSE) {
    if ($cached && $cache = AutolinkCache::get()) {
      return $cache;
    }
    else {
      $links = array();
      $this->maker = new Autolink();
      foreach (node_type_get_names() as $type => $name) {
        $links[$type] = $this->fetchDefined($type) + $this->fetchAbstract($type);
      }
      return $links;
    }
  }

  /**
   * Updates the cache for all node types. If the cache is currently
   * set then we simple check to ensure all node types are set as well.
   */
  function checkCache() {
    $links = array();
    if ($cache = AutolinkCache::get()) {
      $this->maker = new Autolink();
      foreach (node_type_get_names() as $type => $name) {
        if (!isset($cache[$type])) {
          $links = $this->fetchDefined($type) + $this->fetchAbstract($type);
        }
      }
    }
    else {
      foreach (node_type_get_names() as $type => $name) {
        $links = $this->fetchDefined($type) + $this->fetchAbstract($type);
      }
    }
    AutolinkCache::$cache = $links;
    AutolinkCache::save();
  }

  /**
   * Gathers and returns an array of all available links based on enabled settings.
   */
  function fetchLinks($node_type) {
    if ($cache = AutolinkCache::get($node_type)) {
      return $cache;
    }
    $this->maker = new Autolink();
    $links = $this->fetchDefined($node_type) + $this->fetchAbstract($node_type);
    AutolinkCache::set($node_type, $links);
    return $links;
  }

  /**
   * Returns an array of defined link objects.
   */
  protected function fetchDefined($node_type) {
    $links = array();
    $available = array();
    $config = autolink_get('config')->fetchArraySetting('link_types');
    foreach ($this->_info['link types'] as $type) {
      if ($config->isEnabled($type)) {
        $available[] = "a.type = '$type'";
      }
    }
    if (!empty($available)) {
      $result = db_query("SELECT a.* FROM {autolink_link} a INNER JOIN {autolink_group} g ON a.group_type = g.type LEFT JOIN {autolink_group_node_type} t ON a.group_type = t.group_type WHERE g.status <> 0 AND t.node_type = :type AND " . implode (' OR ', $available) . "", array(':type' => $node_type));
      foreach ($result as $data) {
        $data = (array)$data;
        AutolinkMapper::prepare($data);
        $links[] = $this->maker->makeLink($data);
      }
    }
    return $links;
  }

  /**
   * Retrieves abstract links from providers.
   *
   * Links are retrieved in an array of sub-arrays which should hold at least
   * keyword and path data. That data is converted into new AutolinkLink objects
   * using the link type class, so link type class methods are still available.
   */
  protected function fetchAbstract($node_type) {
    $links = array();
    $active = autolink_get('LinkTypeInfo')->getActive();
    $config = autolink_get('config')->fetchArraySetting('link_types');
    // Load only plugin supported link types.
    foreach ($active as $type => $info) {
      // Link types must be enabled and not definable or not identifiable.
      if (($info['definable'] == AUTOLINK_DISABLED && $config->isEnabled($type)) || $info['identifiable'] == AUTOLINK_DISABLED) {
        Autolink::includeFile($info['module'], $info['file']);
        $class = $info['provider'];
        if (class_exists($class)) {
          $object = new $class();
          foreach ($object->provideLinks($node_type) as $data) {
            $data['type'] = $type;
            $data['generic'] = TRUE;
            $links[] = $this->maker->makeLink($data);
          }
        }
        else {
          throw new AutolinkException(AutolinkException::ClassFailed);
        }
      }
    }
    return $links;
  }
}

/**
 * Defines the link mapper interface for loading and saving link objects.
 */
interface AutolinkMapperInterface {
  function save(array $data);
  function load($lid);
  function delete($lid);
}

/**
 * Defines the link mapper class, which saves and loads data from the database.
 */
class AutolinkMapper implements AutolinkMapperInterface {

  function __construct() {}

  /**
   * Saves link data to the database.
   */
  function save(array $data) {
    if (!empty($data['lid']) && !empty($data['keyword']) && !empty($data['path'])) {
      module_invoke_all('autolink_update', (object) $data);
      $this->presave($data);
      drupal_write_record('autolink_link', $data, 'lid');
      $status = SAVED_UPDATED;
    }
    elseif (!empty($data['lid'])) {
      $status = $this->delete($data['lid']);
    }
    elseif (!empty($data['type'])) {
      module_invoke_all('autolink_insert', (object) $data);
      $data['keyword'] = check_plain(trim($data['keyword']));
      $this->presave($data);
      drupal_write_record('autolink_link', $data);
      $status = SAVED_NEW;
    }
    return $status;
  }

  /**
   * Prepares data to be saved by storing extra data in serialized array.
   */
  private function presave(array &$data) {
    // Convert the group back to a format appropriate for drupal_write_record().
    // This is done because the field 'group' is not allowed in the db table.
    $data['group_type'] = isset($data['group']) ? $data['group'] : $data['group_type'];
    unset($data['group']);
    // Store values in 'extra' if they are not standard values.
    if (!isset($data['extra'])) {
      $defaults = array('lid', 'type', 'group_type', 'keyword', 'path', 'query', 'method');
      $data['extra'] = array();
      foreach ($data as $key => $value) {
        if (!in_array($key, $defaults)) {
          $data['extra'][$key] = $value;
          unset($data[$key]);
        }
      }
    }
    $data['extra'] = serialize($data['extra']);
  }

  /**
   * Returns an array of link data.
   */
  function load($lid) {
    try {
      $data = self::fetchData($lid);
      self::prepare($data);
      return $data;
    }
    catch (AutolinkException $e) {
      $e->displayMessage();
    }
  }

  /**
   * Queries the database to retrieve link data.
   */
  protected static function fetchData($lid) {
    $data = db_select('autolink_link', 'a')
      ->fields('a')->condition('lid', $lid, '=')
      ->execute()->fetchObject();
    if (!empty($data)) {
      $data = (array)$data;
      $result = db_query("SELECT node_type FROM {autolink_group_node_type} WHERE group_type = :type", array(':type' => $data['group_type']));
      $data['group'] = $data['group_type'];
      unset($data['group_type']);
      $data['node_types'] = array();
      foreach ($result as $type) {
        $data['node_types'][] = $type->node_type;
      }
      return $data;
    }
    else {
      throw new AutolinkException(AutolinkException::DatabaseFetchFailed);
    }
  }

  /**
   * Prepares data from the database for use in link type classes.
   */
  public static function prepare(array &$data) {
    $data['extra'] = !empty($data['extra']) ? unserialize($data['extra']) : array();
    if (!empty($data['extra']) && is_array($data['extra'])) {
      foreach ($data['extra'] as $key => $value) {
        $data[$key] = $value;
      }
    }
    unset($data['extra']);
  }

  /**
   * Deletes a link.
   */
  function delete($lid) {
    module_invoke_all('autolink_delete', autolink_load($lid));
    db_delete('autolink_link')->condition('lid', $lid)->execute();
  }

}

/**
 * Defines the link interface.
 */
interface AutolinkLinkType {
  function save();
  function get($property);
  function getType();
  function getGroup();
  function getKeyword();
  function getPath();
  function getQuery();
  function getInfo();
}

/**
 * Defines the Autolink link type class.
 */
class AutolinkLink implements AutolinkLinkType {
  private $_data = array();
  public $type = NULL;
  public $_info = NULL;
  protected $config;

  /**
   * Standard constructor method for link objects.
   *
   * @param $data
   *   An array of data with which to populate the object's data property.
   * @param $type
   *   The link type of the link object being created.
   * @param $info
   *   An array of link type info as defined in hook_autolink_link_type_info().
   */
  function __construct(array $data, $type, array $info) {
    $this->type = $type;
    $this->_info = $info;
    $this->_data = $data;
    $this->config = autolink_get('config');
  }

  function __destruct() {
    $this->type = NULL;
    $this->_data = array();
    $this->_info = NULL;
  }

  function __get($property) {
    return $this->$property;
  }

  function __set($property, $value) {
    $this->$property = $value;
  }

  /**
   * Saves the link object to the database.
   */
  function save() {
    // Restore the link type to the array of data to save.
    $data = $this->_data;
    $data['type'] = isset($data['type']) ? $data['type'] : $this->getType();
    autolink_get('mapper')->save($data);
  }

  /**
   * Returns a specific data property based on the given string parameter.
   */
  function get($property) {
    return isset($this->_data[$property]) ? $this->_data[$property] : FALSE;
  }

  /**
   * Returns the link type info array.
   */
  function getInfo() {
    return $this->_info;
  }

  /**
   * Returns the link ID.
   */
  function getIdentifier() {
    return isset($this->_data['lid']) ? $this->_data['lid'] : FALSE;
  }

  /**
   * Returns the link type of the object.
   */
  function getType() {
    return $this->type;
  }

  /**
   * Returns the machine-name of the link group.
   */
  function getGroup() {
    if (!isset($this->_data['group_type']) && isset($this->_data['group'])) {
      return $this->_data['group'];
    }
    return $this->_data['group_type'];
  }

  /**
   * Returns the link keyword.
   */
  function getKeyword() {
    return $this->_data['keyword'];
  }

  /**
   * Returns the link path.
   */
  function getPath() {
    return $this->_data['path'];
  }

  /**
   * Returns the link query.
   */
  function getQuery() {
    return $this->_data['query'];
  }

  function getRelationship() {
    return NULL;
  }

  /**
   * The following methods are for example only.
   */

  /**
   * The setAttributes function may be defined by link types. If the plugin
   * 'attributes' setting is not set to AUTOLINK_ABSOLUTE then the link type
   * attributes method will be used if found.
   */
  //function getAttributes() {
  //  return array(
  //     'title' => $this->get('keyword'),
  //  );
  //}

  /**
   * The setText method is used to get the text of a link. The same properties
   * as discussed above apply here.
   */
  //function getText($match) {
  //  return $match;
  //}

}

/**
 * Defines the link provider class for abstract link types.
 */
abstract class AutolinkProvider {
  protected $config;

  function __construct() {
    $this->config = autolink_get('config');
  }

  abstract function provideLinks($type);
}
