<?php

/**
 * @file
 * Provides link type classes for the Autolink Default plugin.
 */

/**
 * Defines the standard link class for defining regular expressions.
 */
class AutolinkDefaultLink extends AutolinkLink {

  /**
   * Returns attributes based on the current link.
   */
  protected function attributes() {
    return array(
      'title' => $this->get('keyword'),
      'style' => $this->config->get('autolink_style', ''),
      'class' => $this->config->get('autolink_class', ''),
      'target' => url_is_external($this->get('path')) ? $this->config->get('external_target', 'none') : $this->config->get('internal_target', 'none'),
      'rel' => $this->config->fetchSetting('external_rel', 1)->isEnabled() && url_is_external($this->get('path')) ? 'nofollow' : '',
    );
  }

  /**
   * Returns a boolean value indicating whether or not the link is case sensitive.
   */
  function getCase() {
    return $this->get('case') == 1;
  }

  /**
   * Returns an array of stored synonyms.
   *
   * This method is currently not being used because of a strange connection
   * timeout error when using the AutolinkLink::get() method in this context.
   */
  function getSynonyms() {
    $synonyms = $this->get('synonyms');
    return !empty($synonyms) ? $synonyms : array();
  }

}

/**
 * Defines the node link processor class.
 */
class AutolinkNodeLink extends AutolinkDefaultLink {

  /**
   * Builds a token title using the link to node if a pattern has been defined.
   */
  function getAttributes() {
    $parts = explode('/', $this->get('path'));
    if (module_exists('token')) {
      $node = node_load($parts[1]);
      if ($node) {
        $attributes = $this->attributes();
        $pattern = $this->config->get('node_title', '[title]');
        $attributes['title'] = !empty($pattern) && module_exists('token') ? token_replace($pattern, array('node' => $node)) : $this->get('keyword');
        return $attributes;
      }
    }
    return $this->attributes();
  }

  function getRelationship() {
    $parts = explode($this->path);
    return $parts[1];
  }

}

/**
 * Defines the user link processor class.
 */
class AutolinkUserLink extends AutolinkDefaultLink {

  /**
   * Builds a Token title using the user to which the link links.
   */
  function getAttributes() {
    $parts = explode('/', $this->get('path'));
    if ($user = user_load(array('uid' => $parts[1])) && module_exists('token')) {
      $attributes = $this->attributes();
      $pattern = $this->config->get('user_title', '[user]');
      $attributes['title'] = !empty($pattern) && module_exists('token') ? token_replace($pattern, array('user' => $user)) : $this->get('keyword');
      return $attributes;
    }
    return $this->attributes();
  }

}

/**
 * Defines the taxonomy term link processor class.
 */
class AutolinkTermLink extends AutolinkDefaultLink {

  /**
   * Returns attributes with a token based title.
   */
  function getAttributes() {
    $parts = explode('/', $this->get('path'));
    if ($term = taxonomy_term_load($parts[2]) && module_exists('token')) {
      $attributes = $this->attributes();
      $pattern =  $this->config->get('term_title', '[cat]');
      $attributes['title'] = !empty($pattern) && module_exists('token') ? token_replace($pattern, array('term' => $term)) : $this->get('keyword');
      return $attributes;
    }
    return $this->attributes();
  }

}
